import { __awaiter } from 'tslib';
import { ɵɵdefineInjectable, ɵɵinject, Injectable } from '@angular/core';
import { DcSccBaseFactory, DcScc } from 'dc-scc';
import { DcEasApi } from 'dc-eas-api';
import { DcDdsObjectModelV1 } from 'dc-dds-api';

class BaseFactory extends DcSccBaseFactory {
    newRequest() {
        return __awaiter(this, void 0, void 0, function* () {
            var reqObj = yield this.getService().newRequest();
            return reqObj;
        });
    }
    clearCache(obj) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof this._s[obj.getGuid()] != "undefined") {
                delete this._s[obj.getGuid()];
            }
            return this;
        });
    }
    getByGuids(guids, cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            var utilObj = this.getService().getDcScc().getUtilities().getFactories().getAsyncGuids();
            return yield utilObj.getByGuids(guids, this, "getByGuid", cbObj, cbMethod);
        });
    }
}

class Connection extends BaseFactory {
    constructor() {
        super(...arguments);
        this._isReconn = false;
        this._isAuth = false;
        this._authProm = null;
        this._wrkObj = null;
    }
    isAuth() {
        return this._isAuth;
    }
    authMessage() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isAuth() === false) {
                yield this.easAuth();
            }
            var cObj = yield this.getClient();
            return cObj.newMessage();
        });
    }
    unauthMessage() {
        return __awaiter(this, void 0, void 0, function* () {
            var cObj = yield this.getClient();
            return cObj.newMessage();
        });
    }
    getClient() {
        return __awaiter(this, void 0, void 0, function* () {
            var wrkObj = yield this.getWorker();
            return wrkObj.getClient();
        });
    }
    getWorker() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._wrkObj === null) {
                var envFact = this.getMtm().get("DcScc").getEnvironment();
                var hostname = yield envFact.getEnvAsync("dc-ras-api.host");
                var port = yield envFact.getEnvAsync("dc-ras-api.port");
                var ssl = true;
                if ((yield envFact.getEnvAsync("dc-ras-api.tls")) == 0) {
                    ssl = false;
                }
                this._wrkObj = this.getService().getDcScc().getWorkers().getV3(hostname, port, ssl);
                yield this._wrkObj.initialize();
            }
            return this._wrkObj;
        });
    }
    easAuth() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._authProm === null) {
                this._authProm = this.getMtm().get("utilities").getAsync().getPromises().getInline();
                try {
                    var wrkObj = yield this.getWorker();
                    var cObj = wrkObj.getClient();
                    var reqObj = cObj.newMessage();
                    reqObj.setL1("Authentication").setL2("Get").setL3("Salt");
                    var salt = yield reqObj.getResponse(true);
                    var reqObj = cObj.newMessage();
                    reqObj.setL1("Authentication").setL2("Validate").setL3("ByEAS").setL4("EphemeralKey");
                    var hash = yield this.getMtm().get("utilities").getHashing().getSHA().get256("RAS--" + salt);
                    var ephKey = yield this.getMtm().get("DcEasApi").getEphemeralKeys().newReadUserPublic();
                    yield ephKey.signRequest(reqObj, hash);
                    var sessObj = yield reqObj.getResponse(true);
                    if (typeof sessObj["session-id"] != "undefined"
                        && typeof sessObj["session-key"] != "undefined") {
                        cObj.setSession(sessObj["session-id"], sessObj["session-key"]);
                        wrkObj.setTermCb(this, "reconnect");
                        this._isAuth = true;
                        this._authProm["resolve"]();
                    }
                    else {
                        throw this.getMtm().newException("Server authentication returned an invalid session");
                    }
                }
                catch (errObj) {
                    this._isAuth = false;
                    this._wrkObj = null;
                    this._authProm["reject"](errObj);
                    this._authProm = null;
                }
            }
            return this._authProm.promise;
        });
    }
    reconnect(wrkObj) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._isReconn === false) {
                this._isReconn = true;
                this.logInfo("RAS Reconnecting");
                try {
                    try {
                        this._authProm = null;
                        this._isAuth = false;
                        this._wrkObj = null;
                        yield this.easAuth();
                        this._isReconn = false;
                        this.logInfo("RAS Reconnect success");
                    }
                    catch (errObj) {
                        throw this.getService().getMtm().newException("RAS Reconnect failed");
                    }
                }
                catch (errObj) {
                    this._isReconn = false;
                    this.callException(errObj);
                }
            }
        });
    }
}

class Base extends DcDdsObjectModelV1 {
    constructor() {
        super(...arguments);
        this._guid = null;
    }
    baseConstruct(apiObj) {
        return __awaiter(this, void 0, void 0, function* () {
            this.setExceptionCb(this.getService().getLogging(), "exception");
            this._guid = apiObj.guid;
            if (typeof apiObj["ddsId"] != "undefined") {
                var ddsObj = yield this.getService().getDdsClient();
                this.setConfig(ddsObj, apiObj.ddsId, apiObj.ddsAuth);
                var storeObj = yield this.pubStore();
                yield storeObj.initialize();
            }
        });
    }
    getGuid() {
        return this._guid;
    }
    newRequest() {
        return __awaiter(this, void 0, void 0, function* () {
            var reqObj = yield this.getService().newRequest();
            return reqObj;
        });
    }
}

class Network extends Base {
    construct(apiObj) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.baseConstruct(apiObj);
            this.setInit();
            return this;
        });
    }
    getNs() {
        return this.getPubKey("ns", true);
    }
    setNs(nsObj) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                var nsStr = yield nsObj.getExtendedNamespace();
                if (nsStr !== this.getNs()) {
                    var reqObj = yield this.getService().newRequest();
                    reqObj.setL1("Networks").setL2("Set").setL3("Namespace");
                    reqObj.addReq("guid", this.getGuid());
                    reqObj.addReq("unr-ns-guid", nsObj.getGuid());
                    var waitUp = this.getPubUpdateWait();
                    try {
                        yield reqObj.getResponse(true);
                        yield waitUp;
                    }
                    catch (errObj) {
                        this.pubUpdateClear(waitUp);
                        throw errObj;
                    }
                }
                return this;
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
    getDisplayName() {
        return this.getPubKey("displayName", true);
    }
    setDisplayName(name) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.getDisplayName() !== name) {
                var reqObj = yield this.getService().newRequest();
                reqObj.setL1("Networks").setL2("Set").setL3("DisplayName");
                reqObj.addReq("guid", this.getGuid());
                reqObj.addReq("name", name);
                var waitUp = this.getPubUpdateWait();
                try {
                    yield reqObj.getResponse(true);
                    yield waitUp;
                }
                catch (errObj) {
                    this.pubUpdateClear(waitUp);
                    throw errObj;
                }
            }
            return this;
        });
    }
    getWiredRejectVlan() {
        return this.getPubKey("wiredRejectVlan", true);
    }
    setWiredRejectVlan(id) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.getWiredRejectVlan() !== id) {
                var reqObj = yield this.getService().newRequest();
                reqObj.setL1("Networks").setL2("Set").setL3("WiredRejectVlan");
                reqObj.addReq("guid", this.getGuid());
                reqObj.addReq("id", id);
                var waitUp = this.getPubUpdateWait();
                try {
                    yield reqObj.getResponse(true);
                    yield waitUp;
                }
                catch (errObj) {
                    this.pubUpdateClear(waitUp);
                    throw errObj;
                }
            }
            return this;
        });
    }
    getWirelessRejectVlan() {
        return this.getPubKey("wlanRejectVlan", true);
    }
    setWirelessRejectVlan(id) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.getWirelessRejectVlan() !== id) {
                var reqObj = yield this.getService().newRequest();
                reqObj.setL1("Networks").setL2("Set").setL3("WirelessRejectVlan");
                reqObj.addReq("guid", this.getGuid());
                reqObj.addReq("id", id);
                var waitUp = this.getPubUpdateWait();
                try {
                    yield reqObj.getResponse(true);
                    yield waitUp;
                }
                catch (errObj) {
                    this.pubUpdateClear(waitUp);
                    throw errObj;
                }
            }
            return this;
        });
    }
    getNasDevices(cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.getService().getNasDevices().getByNetwork(this, cbObj, cbMethod);
        });
    }
    addNasDevice(nsObj, displayName, srcAddr, secret, ingressPort) {
        return __awaiter(this, void 0, void 0, function* () {
            var reqObj = yield this.getService().newRequest();
            reqObj.setL1("NasDevices").setL2("Add");
            reqObj.addReq("guid", this.getGuid());
            reqObj.addReq("unr-ns-guid", nsObj.getGuid());
            reqObj.addReq("display-name", displayName);
            reqObj.addReq("src-addr", srcAddr);
            reqObj.addReq("password", secret);
            reqObj.addReq("ingress-port", ingressPort);
            var apiObj = yield reqObj.getResponse(true);
            return yield this.getService().getNasDevices().getByApiObj(apiObj);
        });
    }
    getUserMaps(cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.getService().getUserNetMaps().getByNetwork(this, cbObj, cbMethod);
        });
    }
    clearUserMap(mapObj) {
        return __awaiter(this, void 0, void 0, function* () {
            var netObj = yield mapObj.getNetwork();
            if (netObj.getGuid() !== this.getGuid()) {
                throw this.getMtm().newException("Cannot delete, user map is not mapped");
            }
            return this;
        });
    }
    getCreated() {
        return this.getPubKey("created", true);
    }
    delete() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isDel() === false) {
                var reqObj = yield this.getService().newRequest();
                reqObj.setL1("Networks").setL2("Delete").setL3("ByGuid");
                reqObj.addReq("guid", this.getGuid());
                var waitDel = this.getPubDeleteWait();
                try {
                    yield reqObj.getResponse(true);
                    this.setDel();
                    yield waitDel;
                    this.terminate();
                }
                catch (errObj) {
                    this.pubDeleteClear();
                    throw errObj;
                }
            }
            return this;
        });
    }
    terminate(throwOnErr = false) {
        this.setTerm();
        this.getService().getNetworks().clearCache(this);
    }
}

class Networks extends BaseFactory {
    newObj() {
        var rObj = new Network();
        rObj.setService(this.getService());
        return rObj;
    }
    getByApiObj(apiObj) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof this._s[apiObj.guid] === "undefined") {
                this._s[apiObj.guid] = this.newObj();
                this._s[apiObj.guid].construct(apiObj);
            }
            yield this._s[apiObj.guid].waitInit();
            return this._s[apiObj.guid];
        });
    }
    getBySession(cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            //get all networks this user has access to
            var reqObj = yield this.newRequest();
            reqObj.setL1("Networks").setL2("Get").setL3("BySession");
            try {
                var guids = yield reqObj.getResponse(true);
                return yield this.getByGuids(guids, cbObj, cbMethod);
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
    getByGuid(guid, cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof this._s[guid] !== "undefined") {
                yield this._s[guid].waitInit();
                if (cbObj !== null && cbMethod !== null) {
                    cbObj[cbMethod](this._s[guid]);
                }
                return this._s[guid];
            }
            var reqObj = yield this.newRequest();
            reqObj.setL1("Networks").setL2("Get").setL3("ByGuid");
            reqObj.addReq("guid", guid);
            try {
                var apiObj = yield reqObj.getResponse(true);
                var rObj = yield this.getByApiObj(apiObj);
                if (cbObj !== null && cbMethod !== null) {
                    cbObj[cbMethod](rObj);
                }
                return rObj;
            }
            catch (errObj) {
                throw errObj;
            }
        });
    }
    add(nsObj, displayName) {
        return __awaiter(this, void 0, void 0, function* () {
            var reqObj = yield this.newRequest();
            reqObj.setL1("Networks").setL2("Add");
            reqObj.addReq("displayName", displayName);
            reqObj.addReq("unr-ns-guid", nsObj.getGuid());
            try {
                var apiObj = yield reqObj.getResponse(true);
                return yield this.getByApiObj(apiObj);
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
}

class NasDevice extends Base {
    construct(apiObj) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.baseConstruct(apiObj);
            this.setInit();
            return this;
        });
    }
    getNs() {
        return this.getPubKey("ns", true);
    }
    setNs(nsObj) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                var nsStr = yield nsObj.getExtendedNamespace();
                if (nsStr !== this.getNs()) {
                    var reqObj = yield this.getService().newRequest();
                    reqObj.setL1("NasDevices").setL2("Set").setL3("Namespace");
                    reqObj.addReq("guid", this.getGuid());
                    reqObj.addReq("unr-ns-guid", nsObj.getGuid());
                    var waitUp = this.getPubUpdateWait();
                    try {
                        yield reqObj.getResponse(true);
                        yield waitUp;
                    }
                    catch (errObj) {
                        this.pubUpdateClear(waitUp);
                        throw errObj;
                    }
                }
                return this;
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
    getDisplayName() {
        return this.getPubKey("displayName", true);
    }
    setDisplayName(name) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (name != this.getDisplayName()) {
                    var reqObj = yield this.getService().newRequest();
                    reqObj.setL1("NasDevices").setL2("Set").setL3("DisplayName");
                    reqObj.addReq("guid", this.getGuid());
                    reqObj.addReq("name", name);
                    var waitUp = this.getPubUpdateWait();
                    try {
                        yield reqObj.getResponse(true);
                        yield waitUp;
                    }
                    catch (errObj) {
                        this.pubUpdateClear(waitUp);
                        throw errObj;
                    }
                }
                return this;
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
    getIngressPort() {
        return this.getPubKey("ingressPort", true);
    }
    setIngressPort(port) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (port != this.getIngressPort()) {
                    var reqObj = yield this.getService().newRequest();
                    reqObj.setL1("NasDevices").setL2("Set").setL3("IngressPort");
                    reqObj.addReq("guid", this.getGuid());
                    reqObj.addReq("port", port);
                    var waitUp = this.getPubUpdateWait();
                    try {
                        yield reqObj.getResponse(true);
                        yield waitUp;
                    }
                    catch (errObj) {
                        this.pubUpdateClear(waitUp);
                        throw errObj;
                    }
                }
                return this;
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
    getSrcAddress() {
        return this.getPubKey("srcAddr", true);
    }
    setSrcAddress(ip) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (ip != this.getSrcAddress()) {
                    var reqObj = yield this.getService().newRequest();
                    reqObj.setL1("NasDevices").setL2("Set").setL3("SrcAddress");
                    reqObj.addReq("guid", this.getGuid());
                    reqObj.addReq("src-addr", ip);
                    var waitUp = this.getPubUpdateWait();
                    try {
                        yield reqObj.getResponse(true);
                        yield waitUp;
                    }
                    catch (errObj) {
                        this.pubUpdateClear(waitUp);
                        throw errObj;
                    }
                }
                return this;
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
    setSecret(secret) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                var reqObj = yield this.getService().newRequest();
                reqObj.setL1("NasDevices").setL2("Set").setL3("Secret");
                reqObj.addReq("guid", this.getGuid());
                reqObj.addReq("secret", secret);
                yield reqObj.getResponse(true);
                return this;
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
    getNetwork() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.getService().getNetworks().getByGuid(this.getPubKey("networkGuid", true));
        });
    }
    setNetwork(netObj) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                var curObj = yield this.getNetwork();
                if (netObj.getGuid() !== curObj.getGuid()) {
                    var reqObj = yield this.getService().newRequest();
                    reqObj.setL1("NasDevices").setL2("Set").setL3("Network");
                    reqObj.addReq("guid", this.getGuid());
                    reqObj.addReq("net-guid", netObj.getGuid());
                    var waitUp = this.getPubUpdateWait();
                    try {
                        yield reqObj.getResponse(true);
                        yield waitUp;
                    }
                    catch (errObj) {
                        this.pubUpdateClear(waitUp);
                        throw errObj;
                    }
                }
                return this;
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
    getActiveSessions(cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.getService().getNasSessions().getActiveByNasDevice(this, cbObj, cbMethod);
        });
    }
    getCreated() {
        return this.getPubKey("created", true);
    }
    delete() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isDel() === false) {
                var reqObj = yield this.getService().newRequest();
                reqObj.setL1("NasDevices").setL2("Delete").setL3("ByGuid");
                reqObj.addReq("guid", this.getGuid());
                var waitDel = this.getPubDeleteWait();
                try {
                    yield reqObj.getResponse(true);
                    this.setDel();
                    yield waitDel;
                    this.terminate();
                }
                catch (errObj) {
                    this.pubDeleteClear();
                    throw errObj;
                }
            }
            return this;
        });
    }
    terminate(throwOnErr = false) {
        this.setTerm();
        this.getService().getNasDevices().clearCache(this);
    }
}

class NasDevices extends BaseFactory {
    newObj() {
        var rObj = new NasDevice();
        rObj.setService(this.getService());
        return rObj;
    }
    getByApiObj(apiObj) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof this._s[apiObj.guid] === "undefined") {
                this._s[apiObj.guid] = this.newObj();
                this._s[apiObj.guid].construct(apiObj);
            }
            yield this._s[apiObj.guid].waitInit();
            return this._s[apiObj.guid];
        });
    }
    getByNetwork(netObj, cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            var reqObj = yield this.newRequest();
            reqObj.setL1("NasDevices").setL2("Get").setL3("ByNetwork");
            reqObj.addReq("guid", netObj.getGuid());
            try {
                var guids = yield reqObj.getResponse(true);
                return yield this.getByGuids(guids, cbObj, cbMethod);
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
    getByGuid(guid, cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof this._s[guid] !== "undefined") {
                yield this._s[guid].waitInit();
                if (cbObj !== null && cbMethod !== null) {
                    cbObj[cbMethod](this._s[guid]);
                }
                return this._s[guid];
            }
            var reqObj = yield this.newRequest();
            reqObj.setL1("NasDevices").setL2("Get").setL3("ByGuid");
            reqObj.addReq("guid", guid);
            try {
                var apiObj = yield reqObj.getResponse(true);
                var rObj = yield this.getByApiObj(apiObj);
                if (cbObj !== null && cbMethod !== null) {
                    cbObj[cbMethod](rObj);
                }
                return rObj;
            }
            catch (errObj) {
                throw errObj;
            }
        });
    }
}

class NasSession extends Base {
    construct(apiObj) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.baseConstruct(apiObj);
            this.setInit();
            return this;
        });
    }
    getNasDevice() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.getPubKey("nasGuid", true) !== null) {
                return yield this.getService().getNasDevices().getByGuid(this.getPubKey("nasGuid", true));
            }
            else {
                return null;
            }
        });
    }
    getUser() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.getPubKey("userGuid", true) !== null) {
                return yield this.getService().getUsers().getByGuid(this.getPubKey("userGuid", true));
            }
            else {
                return null;
            }
        });
    }
    getAcctInputOctets() {
        return this.getPubKey("acctOctsIn", true);
    }
    getAcctOutputOctets() {
        return this.getPubKey("acctOctsOut", true);
    }
    getAcctSessionTime() {
        if (this.getPubKey("acctSessTime", true) !== 0) {
            return this.getPubKey("acctSessTime", true);
        }
        else {
            return Math.floor(this.getMtm().get("utilities").getTime().getMicroEpoch() - this.getAcctStart());
        }
    }
    getAcctInterval() {
        return this.getPubKey("acctInt", true);
    }
    getAcctSessionId() {
        return this.getPubKey("acctSessionId", true);
    }
    getAcctStart() {
        return this.getPubKey("acctStart", true);
    }
    getAcctUpdate() {
        return this.getPubKey("acctUpdate", true);
    }
    getAcctStop() {
        return this.getPubKey("acctStop", true);
    }
    getAcctTerminationCause() {
        return this.getPubKey("acctTermCause", true);
    }
    getAcctUniqueId() {
        return this.getPubKey("acctUniqueId", true);
    }
    getCalledStationId() {
        return this.getPubKey("calledStationId", true);
    }
    getCallingStationId() {
        return this.getPubKey("callingStationId", true);
    }
    getConnectInfoStart() {
        return this.getPubKey("connInfoStart", true);
    }
    getConnectInfoStop() {
        return this.getPubKey("connInfoStop", true);
    }
    getFramedIpAddress() {
        return this.getPubKey("framedIpAddress", true);
    }
    getFramedProtocol() {
        return this.getPubKey("framedProtocol", true);
    }
    getPortId() {
        return this.getPubKey("portId", true);
    }
    getPortType() {
        return this.getPubKey("portType", true);
    }
    getServiceType() {
        return this.getPubKey("serviceType", true);
    }
    terminate(throwOnErr = false) {
        this.setTerm();
        this.getService().getNasSessions().clearCache(this);
    }
}

class NasSessions extends BaseFactory {
    newObj() {
        var rObj = new NasSession();
        rObj.setService(this.getService());
        return rObj;
    }
    getByApiObj(apiObj) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof this._s[apiObj.guid] === "undefined") {
                this._s[apiObj.guid] = this.newObj();
                this._s[apiObj.guid].construct(apiObj);
            }
            yield this._s[apiObj.guid].waitInit();
            return this._s[apiObj.guid];
        });
    }
    getLastByUser(userObj) {
        return __awaiter(this, void 0, void 0, function* () {
            var reqObj = yield this.newRequest();
            reqObj.setL1("NasSessions").setL2("Get").setL3("Last").setL4("ByUser");
            reqObj.addReq("guid", userObj.getGuid());
            try {
                var apiObj = yield reqObj.getResponse(true);
                if (apiObj !== null) {
                    return yield this.getByApiObj(apiObj);
                }
                else {
                    //user has no sessions
                    return null;
                }
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
    getActiveByNasDevice(nasObj, cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            var reqObj = yield this.newRequest();
            reqObj.setL1("NasSessions").setL2("Get").setL3("Active").setL4("ByNasDevice");
            reqObj.addReq("guid", nasObj.getGuid());
            try {
                var guids = yield reqObj.getResponse(true);
                return yield this.getByGuids(guids, cbObj, cbMethod);
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
    getByGuid(guid, cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof this._s[guid] !== "undefined") {
                yield this._s[guid].waitInit();
                if (cbObj !== null && cbMethod !== null) {
                    cbObj[cbMethod](this._s[guid]);
                }
                return this._s[guid];
            }
            var reqObj = yield this.newRequest();
            reqObj.setL1("NasSessions").setL2("Get").setL3("ByGuid");
            reqObj.addReq("guid", guid);
            try {
                var apiObj = yield reqObj.getResponse(true);
                var rObj = yield this.getByApiObj(apiObj);
                if (cbObj !== null && cbMethod !== null) {
                    cbObj[cbMethod](rObj);
                }
                return rObj;
            }
            catch (errObj) {
                throw errObj;
            }
        });
    }
}

class User extends Base {
    construct(apiObj) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.baseConstruct(apiObj);
            this.setInit();
            return this;
        });
    }
    getUsername() {
        //some mac addresses are all digits, javascript casts to int even if the JSON is a string
        return String(this.getPubKey("username", true));
    }
    setUsername(val) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (val != this.getUsername()) {
                    var reqObj = yield this.getService().newRequest();
                    reqObj.setL1("Users").setL2("Set").setL3("Username");
                    reqObj.addReq("guid", this.getGuid());
                    reqObj.addReq("username", val);
                    var waitUp = this.getPubUpdateWait();
                    try {
                        yield reqObj.getResponse(true);
                        yield waitUp;
                    }
                    catch (errObj) {
                        this.pubUpdateClear(waitUp);
                        throw errObj;
                    }
                }
                return this;
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
    setPassword(val) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                var reqObj = yield this.getService().newRequest();
                reqObj.setL1("Users").setL2("Set").setL3("Password");
                reqObj.addReq("guid", this.getGuid());
                reqObj.addReq("password", val);
                yield reqObj.getResponse(true);
                return this;
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
    getNs() {
        return this.getPubKey("ns", true);
    }
    setNs(nsObj) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                var nsStr = yield nsObj.getExtendedNamespace();
                if (nsStr !== this.getNs()) {
                    var reqObj = yield this.getService().newRequest();
                    reqObj.setL1("Users").setL2("Set").setL3("Namespace");
                    reqObj.addReq("guid", this.getGuid());
                    reqObj.addReq("unr-ns-guid", nsObj.getGuid());
                    var waitUp = this.getPubUpdateWait();
                    try {
                        yield reqObj.getResponse(true);
                        yield waitUp;
                    }
                    catch (errObj) {
                        this.pubUpdateClear(waitUp);
                        throw errObj;
                    }
                }
                return this;
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
    getNote() {
        return this.getPubKey("note", true);
    }
    setNote(val) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (val != this.getNote()) {
                    var reqObj = yield this.getService().newRequest();
                    reqObj.setL1("Users").setL2("Set").setL3("Note");
                    reqObj.addReq("guid", this.getGuid());
                    reqObj.addReq("note", val);
                    var waitUp = this.getPubUpdateWait();
                    try {
                        yield reqObj.getResponse(true);
                        yield waitUp;
                    }
                    catch (errObj) {
                        this.pubUpdateClear(waitUp);
                        throw errObj;
                    }
                }
                return this;
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
    getNetworkMaps(cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.getService().getUserNetMaps().getByUser(this, cbObj, cbMethod);
        });
    }
    clearNetworkMap(mapObj) {
        return __awaiter(this, void 0, void 0, function* () {
            var userObj = yield mapObj.getUser();
            if (userObj.getGuid() !== this.getGuid()) {
                throw this.getMtm().newException("Cannot delete, user map is not mapped");
            }
            return this;
        });
    }
    getLastNasSession() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.getService().getNasSessions().getLastByUser(this);
        });
    }
    getCreated() {
        return this.getPubKey("created", true);
    }
    delete() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isDel() === false) {
                var mapObjs = yield this.getNetworkMaps();
                for (let mapObj of mapObjs) {
                    yield mapObj.delete();
                }
                var reqObj = yield this.getService().newRequest();
                reqObj.setL1("Users").setL2("Delete").setL3("ByGuid");
                reqObj.addReq("guid", this.getGuid());
                var waitDel = this.getPubDeleteWait();
                try {
                    yield reqObj.getResponse(true);
                    this.setDel();
                    yield waitDel;
                    this.terminate();
                }
                catch (errObj) {
                    this.pubDeleteClear();
                    throw errObj;
                }
            }
            return this;
        });
    }
    terminate(throwOnErr = false) {
        this.setTerm();
        this.getService().getUsers().clearCache(this);
    }
}

class Users extends BaseFactory {
    newObj() {
        var rObj = new User();
        rObj.setService(this.getService());
        return rObj;
    }
    getByApiObj(apiObj) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof this._s[apiObj.guid] === "undefined") {
                this._s[apiObj.guid] = this.newObj();
                this._s[apiObj.guid].construct(apiObj);
            }
            yield this._s[apiObj.guid].waitInit();
            return this._s[apiObj.guid];
        });
    }
    getBySession(cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            //get all networks this user has access to
            var reqObj = yield this.newRequest();
            reqObj.setL1("Users").setL2("Get").setL3("BySession");
            try {
                var guids = yield reqObj.getResponse(true);
                return yield this.getByGuids(guids, cbObj, cbMethod);
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
    getByGuid(guid, cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof this._s[guid] !== "undefined") {
                yield this._s[guid].waitInit();
                if (cbObj !== null && cbMethod !== null) {
                    cbObj[cbMethod](this._s[guid]);
                }
                return this._s[guid];
            }
            var reqObj = yield this.newRequest();
            reqObj.setL1("Users").setL2("Get").setL3("ByGuid");
            reqObj.addReq("guid", guid);
            try {
                var apiObj = yield reqObj.getResponse(true);
                if (typeof this._s[apiObj.guid] === "undefined") {
                    this._s[apiObj.guid] = this.newObj();
                    this._s[apiObj.guid].construct(apiObj);
                }
                yield this._s[apiObj.guid].waitInit();
                if (cbObj !== null && cbMethod !== null) {
                    cbObj[cbMethod](this._s[apiObj.guid]);
                }
                return this._s[apiObj.guid];
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
    add(nsObj, username, password) {
        return __awaiter(this, void 0, void 0, function* () {
            var reqObj = yield this.newRequest();
            reqObj.setL1("Users").setL2("Add").setL3("ByUserPass");
            reqObj.addReq("username", username);
            reqObj.addReq("password", password);
            reqObj.addReq("unr-ns-guid", nsObj.getGuid());
            try {
                var apiObj = yield reqObj.getResponse(true);
                return yield this.getByApiObj(apiObj);
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
    getGeneratedUsername() {
        return __awaiter(this, void 0, void 0, function* () {
            var reqObj = yield this.newRequest();
            reqObj.setL1("Users").setL2("Get").setL3("Generated").setL4("Username");
            try {
                return yield reqObj.getResponse(true);
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
    getGeneratedPassword(length = 12) {
        return __awaiter(this, void 0, void 0, function* () {
            var reqObj = yield this.newRequest();
            reqObj.setL1("Users").setL2("Get").setL3("Generated").setL4("Password");
            reqObj.addReq("length", length);
            try {
                return yield reqObj.getResponse(true);
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
}

class UserNetMap extends Base {
    construct(apiObj) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.baseConstruct(apiObj);
            this.setInit();
            return this;
        });
    }
    getUser(cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.getService().getUsers().getByGuid(this.getPubKey("userGuid", true), cbObj, cbMethod);
        });
    }
    getNetwork(cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.getService().getNetworks().getByGuid(this.getPubKey("networkGuid", true), cbObj, cbMethod);
        });
    }
    getAttrs(cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.getService().getUserNetMapAttrs().getByMap(this, cbObj, cbMethod);
        });
    }
    addAttr(typeObj, value) {
        return __awaiter(this, void 0, void 0, function* () {
            var attrObj = yield this.getAttrByType(typeObj, false);
            if (attrObj !== null) {
                throw this.getMtm().newException("Cannot add attribute exists");
            }
            var reqObj = yield this.getService().newRequest();
            reqObj.setL1("UserNetMapAttrs").setL2("Add");
            reqObj.addReq("map-guid", this.getGuid());
            reqObj.addReq("type-guid", typeObj.getGuid());
            reqObj.addReq("value", value);
            var apiObj = yield reqObj.getResponse(true);
            return yield this.getService().getUserNetMapAttrs().getByApiObj(apiObj);
        });
    }
    getAttrByType(typeObj, throwOnErr = false) {
        return __awaiter(this, void 0, void 0, function* () {
            var attrObjs = yield this.getAttrs();
            for (let attrObj of attrObjs) {
                var eType = yield attrObj.getType();
                if (eType.getGuid() === typeObj.getGuid()) {
                    return attrObj;
                }
            }
            if (throwOnErr === true) {
                throw this.getMtm().newException("Attribute type is not mapped");
            }
            else {
                return null;
            }
        });
    }
    setAttr(typeObj, value) {
        return __awaiter(this, void 0, void 0, function* () {
            var attrObj = yield this.getAttrByType(typeObj, false);
            if (attrObj === null) {
                attrObj = yield this.addAttr(typeObj, value);
            }
            else {
                yield attrObj.setValue(value);
            }
            return attrObj;
        });
    }
    clearAttr(attrObj) {
        return __awaiter(this, void 0, void 0, function* () {
            var mapObj = yield attrObj.getMap();
            if (mapObj.getGuid() !== this.getGuid()) {
                throw this.getMtm().newException("Cannot delete, attribute is not mapped");
            }
            return this;
        });
    }
    getEnabled() {
        return this.getPubKey("enabled", true);
    }
    setEnabled(bool) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (bool !== this.getEnabled()) {
                    var reqObj = yield this.getService().newRequest();
                    reqObj.setL1("UserNetMaps").setL2("Set").setL3("Enabled");
                    reqObj.addReq("guid", this.getGuid());
                    reqObj.addReq("enabled", bool);
                    var waitUp = this.getPubUpdateWait();
                    try {
                        yield reqObj.getResponse(true);
                        yield waitUp;
                    }
                    catch (errObj) {
                        this.pubUpdateClear(waitUp);
                        throw errObj;
                    }
                }
                return this;
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
    getCreated() {
        return this.getPubKey("created", true);
    }
    delete() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isDel() === false) {
                var netObj = yield this.getNetwork();
                netObj.clearUserMap(this);
                var userObj = yield this.getUser();
                userObj.clearNetworkMap(this);
                var reqObj = yield this.getService().newRequest();
                reqObj.setL1("UserNetMaps").setL2("Delete").setL3("ByGuid");
                reqObj.addReq("guid", this.getGuid());
                var waitDel = this.getPubDeleteWait();
                try {
                    yield reqObj.getResponse(true);
                    this.setDel();
                    yield waitDel;
                    this.terminate();
                }
                catch (errObj) {
                    this.pubDeleteClear();
                    throw errObj;
                }
            }
            return this;
        });
    }
    terminate(throwOnErr = false) {
        this.setTerm();
        this.getService().getUserNetMaps().clearCache(this);
    }
}

class UserNetMaps extends BaseFactory {
    newObj() {
        var rObj = new UserNetMap();
        rObj.setService(this.getService());
        return rObj;
    }
    getByApiObj(apiObj) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof this._s[apiObj.guid] === "undefined") {
                this._s[apiObj.guid] = this.newObj();
                this._s[apiObj.guid].construct(apiObj);
            }
            yield this._s[apiObj.guid].waitInit();
            return this._s[apiObj.guid];
        });
    }
    getByNetwork(netObj, cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            var sTime = this.getMtm().get("utilities").getTime().getMicroEpoch();
            var reqObj = yield this.newRequest();
            reqObj.setL1("UserNetMaps").setL2("Get").setL3("ByNetwork");
            reqObj.addReq("guid", netObj.getGuid());
            try {
                var guids = yield reqObj.getResponse(true);
                return yield this.getByGuids(guids, cbObj, cbMethod);
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
    getByUser(userObj, cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            var sTime = this.getMtm().get("utilities").getTime().getMicroEpoch();
            var reqObj = yield this.newRequest();
            reqObj.setL1("UserNetMaps").setL2("Get").setL3("ByUser");
            reqObj.addReq("guid", userObj.getGuid());
            try {
                var guids = yield reqObj.getResponse(true);
                return yield this.getByGuids(guids, cbObj, cbMethod);
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
    getByGuid(guid, cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof this._s[guid] !== "undefined") {
                yield this._s[guid].waitInit();
                if (cbObj !== null && cbMethod !== null) {
                    cbObj[cbMethod](this._s[guid]);
                }
                return this._s[guid];
            }
            var reqObj = yield this.newRequest();
            reqObj.setL1("UserNetMaps").setL2("Get").setL3("ByGuid");
            reqObj.addReq("guid", guid);
            try {
                var apiObj = yield reqObj.getResponse(true);
                var rObj = yield this.getByApiObj(apiObj);
                if (cbObj !== null && cbMethod !== null) {
                    cbObj[cbMethod](rObj);
                }
                return rObj;
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
    add(userObj, netObj, cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            var reqObj = yield this.newRequest();
            reqObj.setL1("UserNetMaps").setL2("Add");
            reqObj.addReq("user-guid", userObj.getGuid());
            reqObj.addReq("net-guid", netObj.getGuid());
            try {
                var apiObj = yield reqObj.getResponse(true);
                if (typeof this._s[apiObj.guid] === "undefined") {
                    this._s[apiObj.guid] = this.newObj();
                    this._s[apiObj.guid].construct(apiObj);
                }
                yield this._s[apiObj.guid].waitInit();
                if (cbObj !== null && cbMethod !== null) {
                    cbObj[cbMethod](this._s[apiObj.guid]);
                }
                return this._s[apiObj.guid];
            }
            catch (errObj) {
                throw errObj;
            }
        });
    }
}

class UserNetMapAttr extends Base {
    construct(apiObj) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.baseConstruct(apiObj);
            this.setInit();
            return this;
        });
    }
    getMap() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.getService().getUserNetMaps().getByGuid(this.getPubKey("mapGuid", true));
        });
    }
    getType() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.getService().getAttrTypes().getByGuid(this.getPubKey("typeGuid", true));
        });
    }
    getValue() {
        return this.getPubKey("value", true);
    }
    setValue(val) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.getValue() !== val) {
                var reqObj = yield this.getService().newRequest();
                reqObj.setL1("UserNetMapAttrs").setL2("Set").setL3("Value");
                reqObj.addReq("guid", this.getGuid());
                reqObj.addReq("value", val);
                var waitUp = this.getPubUpdateWait();
                try {
                    yield reqObj.getResponse(true);
                    yield waitUp;
                }
                catch (errObj) {
                    this.pubUpdateClear(waitUp);
                    throw errObj;
                }
            }
            return this;
        });
    }
    getCreated() {
        return this.getPubKey("created", true);
    }
    delete() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isDel() === false) {
                var mapObj = yield this.getMap();
                mapObj.clearAttr(this);
                var reqObj = yield this.getService().newRequest();
                reqObj.setL1("UserNetMapAttrs").setL2("Delete").setL3("ByGuid");
                reqObj.addReq("guid", this.getGuid());
                var waitDel = this.getPubDeleteWait();
                try {
                    yield reqObj.getResponse(true);
                    this.setDel();
                    yield waitDel;
                    this.terminate();
                }
                catch (errObj) {
                    this.pubDeleteClear();
                    throw errObj;
                }
            }
            return this;
        });
    }
    terminate(throwOnErr = false) {
        this.setTerm();
        this.getService().getUserNetMapAttrs().clearCache(this);
    }
}

class UserNetMapAttrs extends BaseFactory {
    newObj() {
        var rObj = new UserNetMapAttr();
        rObj.setService(this.getService());
        return rObj;
    }
    getByApiObj(apiObj) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof this._s[apiObj.guid] === "undefined") {
                this._s[apiObj.guid] = this.newObj();
                this._s[apiObj.guid].construct(apiObj);
            }
            yield this._s[apiObj.guid].waitInit();
            return this._s[apiObj.guid];
        });
    }
    getByMap(netObj, cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            var reqObj = yield this.newRequest();
            reqObj.setL1("UserNetMapAttrs").setL2("Get").setL3("ByMap");
            reqObj.addReq("guid", netObj.getGuid());
            try {
                var guids = yield reqObj.getResponse(true);
                return yield this.getByGuids(guids, cbObj, cbMethod);
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
    getByGuid(guid, cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof this._s[guid] !== "undefined") {
                yield this._s[guid].waitInit();
                if (cbObj !== null && cbMethod !== null) {
                    cbObj[cbMethod](this._s[guid]);
                }
                return this._s[guid];
            }
            var reqObj = yield this.newRequest();
            reqObj.setL1("UserNetMapAttrs").setL2("Get").setL3("ByGuid");
            reqObj.addReq("guid", guid);
            try {
                var apiObj = yield reqObj.getResponse(true);
                var rObj = yield this.getByApiObj(apiObj);
                if (cbObj !== null && cbMethod !== null) {
                    cbObj[cbMethod](rObj);
                }
                return rObj;
            }
            catch (errObj) {
                throw errObj;
            }
        });
    }
}

class Reports extends BaseFactory {
    downloadCsv(name, str) {
        try {
            const blob = new Blob([str], { type: "text/csv" });
            const url = window.URL.createObjectURL(blob);
            const tempLink = document.createElement('a');
            tempLink.style.display = 'none';
            tempLink.href = url;
            tempLink.setAttribute('download', name + ".csv");
            if (typeof tempLink.download === 'undefined') {
                tempLink.setAttribute('target', '_blank');
            }
            document.body.appendChild(tempLink);
            tempLink.click();
            document.body.removeChild(tempLink);
            setTimeout(() => {
                window.URL.revokeObjectURL(url);
            }, 100);
        }
        catch (errObj) {
            throw errObj;
        }
    }
    getNetworkMapsAllAttrs(netObj) {
        return __awaiter(this, void 0, void 0, function* () {
            var reqObj = yield this.newRequest();
            reqObj.setL1("Reports").setL2("Networks").setL3("Users");
            reqObj.setL4("Maps").setL5("Attrs").setL6("All");
            reqObj.addReq("guid", netObj.getGuid());
            try {
                var csvData = yield reqObj.getResponse(true);
                this.downloadCsv(netObj.getDisplayName() + "-network-attrs", csvData);
            }
            catch (errObj) {
                throw errObj;
            }
        });
    }
}

class AttrType extends Base {
    construct(apiObj) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.baseConstruct(apiObj);
            this.setInit();
            return this;
        });
    }
    getLabel() {
        return this.getPubKey("label", true);
    }
    getDisplayName() {
        return this.getPubKey("displayName", true);
    }
    getName() {
        return this.getPubKey("attrName", true);
    }
    getOperator() {
        return this.getPubKey("operator", true);
    }
    getCreated() {
        return this.getPubKey("created", true);
    }
    terminate(throwOnErr = false) {
        this.setTerm();
        this.getService().getAttrTypes().clearCache(this);
    }
}

class AttrTypes extends BaseFactory {
    newObj() {
        var rObj = new AttrType();
        rObj.setService(this.getService());
        return rObj;
    }
    getByApiObj(apiObj) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof this._s[apiObj.guid] === "undefined") {
                this._s[apiObj.guid] = this.newObj();
                this._s[apiObj.guid].construct(apiObj);
            }
            yield this._s[apiObj.guid].waitInit();
            return this._s[apiObj.guid];
        });
    }
    getAll(cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            var reqObj = yield this.newRequest();
            reqObj.setL1("AttrTypes").setL2("Get").setL3("All");
            try {
                var guids = yield reqObj.getResponse(true);
                return yield this.getByGuids(guids, cbObj, cbMethod);
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
    getByGuid(guid, cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof this._s[guid] !== "undefined") {
                yield this._s[guid].waitInit();
                if (cbObj !== null && cbMethod !== null) {
                    cbObj[cbMethod](this._s[guid]);
                }
                return this._s[guid];
            }
            var reqObj = yield this.newRequest();
            reqObj.setL1("AttrTypes").setL2("Get").setL3("ByGuid");
            reqObj.addReq("guid", guid);
            try {
                var apiObj = yield reqObj.getResponse(true);
                var rObj = yield this.getByApiObj(apiObj);
                if (cbObj !== null && cbMethod !== null) {
                    cbObj[cbMethod](rObj);
                }
                return rObj;
            }
            catch (errObj) {
                throw errObj;
            }
        });
    }
}

class DcRasApi {
    constructor(_dcScc, _dcEasApi) {
        this._dcScc = _dcScc;
        this._dcEasApi = _dcEasApi;
        this._connFact = null;
        this._sessObj = null;
        this._repFact = null;
        this._netFact = null;
        this._nasFact = null;
        this._nassFact = null;
        this._uFact = null;
        this._unmFact = null;
        this._unmaFact = null;
        this._atFact = null;
        this._ddsClient = null;
        this._dcScc.getMtm().set("DcRasApi", this);
    }
    getDcScc() {
        return this._dcScc;
    }
    getMtm() {
        return this.getDcScc().getMtm();
    }
    getLogging() {
        return this.getDcScc().getLogging();
    }
    getConnection() {
        if (this._connFact === null) {
            this._connFact = new Connection();
            this._connFact.setService(this);
            this._connFact.setExceptionCb(this.getLogging(), "exception");
        }
        return this._connFact;
    }
    getNetworks() {
        if (this._netFact === null) {
            this._netFact = new Networks();
            this._netFact.setService(this);
            this._netFact.setExceptionCb(this.getLogging(), "exception");
        }
        return this._netFact;
    }
    getNasDevices() {
        if (this._nasFact === null) {
            this._nasFact = new NasDevices();
            this._nasFact.setService(this);
            this._nasFact.setExceptionCb(this.getLogging(), "exception");
        }
        return this._nasFact;
    }
    getNasSessions() {
        if (this._nassFact === null) {
            this._nassFact = new NasSessions();
            this._nassFact.setService(this);
            this._nassFact.setExceptionCb(this.getLogging(), "exception");
        }
        return this._nassFact;
    }
    getUserNetMaps() {
        if (this._unmFact === null) {
            this._unmFact = new UserNetMaps();
            this._unmFact.setService(this);
            this._unmFact.setExceptionCb(this.getLogging(), "exception");
        }
        return this._unmFact;
    }
    getUserNetMapAttrs() {
        if (this._unmaFact === null) {
            this._unmaFact = new UserNetMapAttrs();
            this._unmaFact.setService(this);
            this._unmaFact.setExceptionCb(this.getLogging(), "exception");
        }
        return this._unmaFact;
    }
    getAttrTypes() {
        if (this._atFact === null) {
            this._atFact = new AttrTypes();
            this._atFact.setService(this);
            this._atFact.setExceptionCb(this.getLogging(), "exception");
        }
        return this._atFact;
    }
    getUsers() {
        if (this._uFact === null) {
            this._uFact = new Users();
            this._uFact.setService(this);
            this._uFact.setExceptionCb(this.getLogging(), "exception");
        }
        return this._uFact;
    }
    getReports() {
        if (this._repFact === null) {
            this._repFact = new Reports();
            this._repFact.setService(this);
            this._repFact.setExceptionCb(this.getLogging(), "exception");
        }
        return this._repFact;
    }
    getDdsClient() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._ddsClient === null) {
                var envFact = this.getMtm().get("DcScc").getEnvironment();
                var hostname = yield envFact.getEnvAsync("dc-dds-api.host");
                var port = yield envFact.getEnvAsync("dc-dds-api.port");
                var ssl = true;
                if ((yield envFact.getEnvAsync("dc-dds-api.tls")) == 0) {
                    ssl = false;
                }
                this._ddsClient = this.getMtm().get("DcDdsApi").getClients().getV1(hostname, port, ssl);
                yield this._ddsClient.initialize();
            }
            return this._ddsClient;
        });
    }
    newRequest() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.getConnection().authMessage();
        });
    }
    rawRequest() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.getConnection().unauthMessage();
        });
    }
}
DcRasApi.ɵprov = ɵɵdefineInjectable({ factory: function DcRasApi_Factory() { return new DcRasApi(ɵɵinject(DcScc), ɵɵinject(DcEasApi)); }, token: DcRasApi, providedIn: "root" });
DcRasApi.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
DcRasApi.ctorParameters = () => [
    { type: DcScc },
    { type: DcEasApi }
];

/*
 * Public API Surface of dc-ras-api
 */

/**
 * Generated bundle index. Do not edit.
 */

export { DcRasApi };
//# sourceMappingURL=dc-ras-api.js.map
